# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .tool import Tool

__all__ = ["ToolUpsertBaseToolsResponse"]

ToolUpsertBaseToolsResponse: TypeAlias = List[Tool]
