# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ProviderType"]

ProviderType: TypeAlias = Literal[
    "anthropic",
    "azure",
    "bedrock",
    "cerebras",
    "deepseek",
    "google_ai",
    "google_vertex",
    "groq",
    "hugging-face",
    "letta",
    "lmstudio_openai",
    "mistral",
    "ollama",
    "openai",
    "together",
    "vllm",
    "xai",
]
